/**
 * Copyright (C) 2014 Deepin Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 **/

package dock

import "pkg.deepin.io/lib/dbus"

// import "flag"
import "github.com/BurntSushi/xgb/xproto"
import "os"
import "path/filepath"
import "errors"
import "fmt"

// EntryManager为驻留程序以及打开程序的管理器。
type EntryManager struct {
	runtimeApps map[string]*RuntimeApp
	normalApps  map[string]*NormalApp
	appEntries  map[string]*AppEntry

	// 保持 entry 的顺序
	entryIDs []string
}

func NewEntryManager() *EntryManager {
	m := &EntryManager{}
	m.runtimeApps = make(map[string]*RuntimeApp)
	m.normalApps = make(map[string]*NormalApp)
	m.appEntries = make(map[string]*AppEntry)
	return m
}

// Reorder 重排序dock上的app项目
// 参数entryIDs为dock上app项目的新顺序id列表，要求与当前app项目是同一个集合，只是顺序不同。
func (m *EntryManager) Reorder(entryIDs []string) error {
	logger.Debugf("Reorder entryIDs %#v", entryIDs)
	if len(entryIDs) != len(m.entryIDs) {
		logger.Warning("Reorder: len(entryIDs) != len(m.entryIDs)")
		return errors.New("length of incomming entryIDs not equal length of m.entryIDs")
	}
	var orderedEntryIDs []string
	for _, id := range entryIDs {
		_, ok := m.appEntries[id]
		if ok {
			orderedEntryIDs = append(orderedEntryIDs, id)
		} else {
			logger.Warningf("Reorder: invaild entry id %q", id)
			return fmt.Errorf("Invaild entry id %q", id)
		}
	}
	m.entryIDs = orderedEntryIDs
	dockManager.dockedAppManager.reorderThenSave(m.entryIDs)
	return nil
}

func (m *EntryManager) GetEntryIDs() []string {
	return m.entryIDs
}

func (m *EntryManager) runtimeAppChanged(xids []xproto.Window) {
	logger.Debug("runtime app changed")
	willBeDestroied := make(map[string]*RuntimeApp)
	for _, app := range m.runtimeApps {
		willBeDestroied[app.Id] = app
	}

	// 1. create newfound RuntimeApps
	for _, xid := range xids {
		if isNormalWindow(xid) {
			// FIXME
			// appId := find_app_id_by_xid(xid, DisplayModeType(dockManager.setting.GetDisplayMode()))
			appId := find_app_id_by_xid(xid, dockManager.displayMode)
			if rApp, ok := m.runtimeApps[appId]; ok {
				logger.Debugf("%s is already existed, attach xid: 0x%x", appId, xid)
				willBeDestroied[appId] = nil
				rApp.attachXid(xid)
			} else {
				m.createRuntimeApp(xid)
			}
		}
	}
	// 2. destroy disappeared RuntimeApps since last runtimeAppChanged point
	for _, app := range willBeDestroied {
		if app != nil {
			m.destroyRuntimeApp(app)
		}
	}
}

func (m *EntryManager) mustGetEntry(nApp *NormalApp, rApp *RuntimeApp) *AppEntry {
	if rApp != nil {
		if e, ok := m.appEntries[rApp.Id]; ok {
			return e
		} else {
			e := NewAppEntryWithRuntimeApp(rApp)
			m.appEntries[rApp.Id] = e
			m.entryIDs = append(m.entryIDs, rApp.Id)
			err := dbus.InstallOnSession(e)
			if err != nil {
				logger.Warning("Install NewRuntimeAppEntry to dbus failed:", err)
			}
			return e
		}
	} else if nApp != nil {
		if e, ok := m.appEntries[nApp.Id]; ok {
			return e
		} else {
			e := NewAppEntryWithNormalApp(nApp)
			m.appEntries[nApp.Id] = e
			m.entryIDs = append(m.entryIDs, nApp.Id)
			err := dbus.InstallOnSession(e)
			if err != nil {
				logger.Warning("Install NewNormalAppEntry to dbus failed:", err)
			}
			return e
		}
	}
	panic("mustGetEntry: at least give one app")
}

func (m *EntryManager) destroyEntry(appId string) {
	if e, ok := m.appEntries[appId]; ok {
		e.detachNormalApp()
		e.detachRuntimeApp()
		dbus.ReleaseName(e)
		dbus.UnInstallObject(e)
		logger.Info("destroyEntry:", appId)
	}
	delete(m.appEntries, appId)
	m.entryIDs = strSliceRemove(m.entryIDs, appId)
}

func strSliceRemove(slice []string, str string) []string {
	var index int = -1
	for i, v := range slice {
		if v == str {
			index = i
		}
	}
	if index != -1 {
		return append(slice[:index], slice[index+1:]...)
	}
	return slice
}

func (m *EntryManager) updateEntry(appId string, nApp *NormalApp, rApp *RuntimeApp) {
	switch {
	case nApp == nil && rApp == nil:
		m.destroyEntry(appId)
	case nApp == nil && rApp != nil:
		e := m.mustGetEntry(nApp, rApp)
		e.attachRuntimeApp(rApp)
		e.detachNormalApp()
	case nApp != nil && rApp != nil:
		e := m.mustGetEntry(nApp, rApp)
		e.attachNormalApp(nApp)
		e.attachRuntimeApp(rApp)
	case nApp != nil && rApp == nil:
		e := m.mustGetEntry(nApp, rApp)
		e.attachNormalApp(nApp)
		e.detachRuntimeApp()
	}
}

func (m *EntryManager) createRuntimeApp(xid xproto.Window) *RuntimeApp {
	appId := find_app_id_by_xid(xid, DisplayModeType(dockManager.setting.GetDisplayMode()))
	if appId == "" {
		logger.Debug("get appid for", xid, "failed")
		return nil
	}

	if v, ok := m.runtimeApps[appId]; ok {
		return v
	}

	rApp := NewRuntimeApp(xid, appId)
	if rApp == nil {
		return nil
	}

	m.runtimeApps[appId] = rApp
	m.updateEntry(appId, m.mustGetEntry(nil, rApp).nApp, rApp)
	return rApp
}
func (m *EntryManager) destroyRuntimeApp(rApp *RuntimeApp) {
	delete(m.runtimeApps, rApp.Id)
	m.updateEntry(rApp.Id, m.mustGetEntry(nil, rApp).nApp, nil)
}
func (m *EntryManager) createNormalApp(id string) {
	logger.Info("createNormalApp for", id)
	if _, ok := m.normalApps[id]; ok {
		logger.Debug("normal app for", id, "is exist")
		return
	}

	desktopId := id + ".desktop"
	nApp := NewNormalApp(desktopId)
	if nApp == nil {
		logger.Info("get desktop file failed, create", id, "from scratch file")
		newId := filepath.Join(
			os.Getenv("HOME"),
			".config/dock/scratch",
			desktopId,
		)
		nApp = NewNormalApp(newId)
		if nApp == nil {
			logger.Warning("create normal app failed:", id)
			dockManager.dockedAppManager.Undock(id)
			return
		}
	}

	m.normalApps[id] = nApp
	m.updateEntry(id, nApp, m.mustGetEntry(nApp, nil).rApp)
}
func (m *EntryManager) destroyNormalApp(id string) {
	if nApp, ok := m.normalApps[id]; ok {
		logger.Debugf("destroyNormalApp id: %q", id)
		delete(m.normalApps, nApp.Id)
		m.updateEntry(nApp.Id, nil, m.mustGetEntry(nApp, nil).rApp)
	} else {
		logger.Debugf("no need destroyNormalApp id: %q", id)
	}
}
